package container;

import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * The customer
 * 
 * @author Valerius Weigandt
 */
public class Customer implements Observer, Serializable {

	/** serialization */
	private static final long serialVersionUID = -3834221211740770137L;

	/**
	 * The empty constructor, for setting {@link #customerID},{@link #password},
	 * {@link #accounts} and {@link #gender} later.
	 */
	public Customer() {
	}

	/**
	 * @param customerID
	 * @param password
	 * @param eMail
	 * @param firstName
	 * @param familyName
	 * @param plz
	 * @param city
	 * @param street
	 * @param telephoneNumber
	 * @param dob
	 * @param accounts
	 * @param gender
	 */
	public Customer(int customerID, String password, String eMail,
			String firstName, String familyName, int plz, String city,
			String street, int telephoneNumber, Date dob,
			List<Account> accounts, Gender gender) {
		this.customerID = customerID;
		this.password = password;
		this.email = eMail;
		this.firstName = firstName;
		this.familyName = familyName;
		this.plz = plz;
		this.city = city;
		this.street = street;
		this.telephoneNumber = telephoneNumber;
		this.dob = dob;
		this.accounts = accounts;
		this.gender = gender;
	}

	/** Unique ID */
	private int customerID;
	/** The password */
	@NotNull(message = "Bitte ein Passwort eingeben!")
	private String password;
	/** E-Mail */
	@Pattern(regexp = "([^.@]+)(\\.[^.@]+)*@([^.@]+\\.)+([^.@]+)", message = "Kein Gültiges Format!")
	private String email;
	/** First name */
	@NotNull(message = "Bitte ein Vornamen eingeben!")
	private String firstName;
	/** Family name */
	@NotNull(message = "Bitte ein Familennamen eingeben!")
	private String familyName;
	/** PLZ of the customer */
	private int plz;
	/** The city */
	private String city;
	/** The street */
	private String street;
	/** The telephone number of the customer */
	private int telephoneNumber;
	/** Day of birth */
	@NotNull(message = "Bitte ein Geburtsdatum angeben")
	private Date dob;
	/** Accounts of the customer */
	private List<Account> accounts; // Set should be synchronized
	/** Gender of the customer */
	private Gender gender;

	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @param password
	 *            the password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @return the gender
	 */
	public Gender getGender() {
		return gender;
	}

	/**
	 * @param gender
	 *            the gender to set
	 */
	public void setGender(Gender gender) {
		this.gender = gender;
	}

	/**
	 * @return the accounts
	 */
	public List<Account> getAccounts() {
		return accounts;
	}

	/**
	 * @param accounts
	 *            the accounts to set
	 */
	public void setAccounts(List<Account> accounts) {
		this.accounts = accounts;
		for (Account account : accounts) {
			account.addObserver(this);
		}
	}

	/**
	 * Adds a new account
	 * 
	 * @param account
	 *            The account to add
	 * */
	public void addAccount(Account account) {
		if (account == null)
			throw new NullPointerException("Account can't be NULL");
		if (this.accounts == null)
			this.accounts = new LinkedList<Account>();
		this.accounts.add(account);
		account.addObserver(this);
	}

	/**
	 * Deletes the account
	 * 
	 * @param account
	 *            The account to delete
	 */
	public void deleteAccount(Account account) {
		if (accounts == null)
			return;
		accounts.remove(account);
	}

	/**
	 * Deletes a account by ID
	 * 
	 * @param accountID
	 *            The account ID
	 */
	public void deleteAccountByID(int accountID) {
		for (Account account : accounts) {
			if (account.getAccountID() == accountID) {
				accounts.remove(accountID);
				return;
			}
		}
	}

	/**
	 * @return the customerID
	 */
	public int getCustomerID() {
		return customerID;
	}

	/**
	 * @param customerID
	 *            the customerID to set
	 */
	public void setCustomerID(int customerID) {
		this.customerID = customerID;
	}

	/**
	 * @return the eMail
	 */
	public String geteMail() {
		return email;
	}

	/**
	 * @param eMail
	 *            the eMail to set
	 */
	public void seteMail(String eMail) {
		this.email = eMail;
	}

	/**
	 * @return the firstName
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * @param firstName
	 *            the firstName to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * @return the familyName
	 */
	public String getFamilyName() {
		return familyName;
	}

	/**
	 * @param familyName
	 *            the familyName to set
	 */
	public void setFamilyName(String familyName) {
		this.familyName = familyName;
	}

	/**
	 * @return the plz
	 */
	public int getPlz() {
		return plz;
	}

	/**
	 * @param plz
	 *            the plz to set
	 */
	public void setPlz(int plz) {
		this.plz = plz;
	}

	/**
	 * @return the city
	 */
	public String getCity() {
		return city;
	}

	/**
	 * @param city
	 *            the city to set
	 */
	public void setCity(String city) {
		this.city = city;
	}

	/**
	 * @return the street
	 */
	public String getStreet() {
		return street;
	}

	/**
	 * @param street
	 *            the street to set
	 */
	public void setStreet(String street) {
		this.street = street;
	}

	/**
	 * @return the telephoneNumber
	 */
	public int getTelephoneNumber() {
		return telephoneNumber;
	}

	/**
	 * @param telephoneNumber
	 *            the telephoneNumber to set
	 */
	public void setTelephoneNumber(int telephoneNumber) {
		this.telephoneNumber = telephoneNumber;
	}

	/**
	 * @return the dob
	 */
	public Date getDob() {
		return dob;
	}

	/**
	 * @param dob
	 *            the dob to set
	 */
	public void setDob(Date dob) {
		this.dob = dob;
	}

	@Override
	public void update(Observable o, Object arg) {

	}
}
