package controller.web;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import globals.BankSettings;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import container.Account;
import container.Customer;
import container.InterestRate;
import container.Transaction;
import container.TransactionType;
import database.Database;
import database.DatabaseEventLogger;
import database.helpers.DatabaseLogEntry;

/**
 * The admin controller for different configuration purposes. Here you can set
 * the interest rates and view logs.
 * 
 * @author Valerius
 * @author Carl
 */
@ManagedBean
@SessionScoped
public class AdminController implements Serializable {

	/** Serialisation */
	private static final long serialVersionUID = 4270909178523460235L;

	/** the interestRate of the current running bank settings */
	private InterestRate interestRate = BankSettings.getInterestRate();

	/** List which contains the whole event log */
	private List<DatabaseLogEntry> events = new ArrayList<DatabaseLogEntry>();

	/** List of all customers and their accounts */
	private List<Customer> customers = new ArrayList<Customer>();

	/** the interest percentage */
	private double interestAmount = BankSettings.getInterest() * 100;

	/** account id where to transmit the money */
	@NotNull(message = "Bitte geben Sie eine Kontonummer ein!")
	@Pattern(regexp = "(\\d){5,}", message = "Bitte geben Sie mindestens 5 Zahlen ein")
	private String recieverAccountID;

	/** the reason of the payment */
	@NotNull(message = "Bitte geben sie ein Verwendungszweck an!")
	private String paymentReason;

	/** the amount to transmit */
	@Min(0)
	@NotNull(message = "Bitte geben sie ein Betrag an!")
	private BigDecimal amount;

	/** From Date for log filter */
	private Date logFrom;

	/** Until Date for log filter */
	private Date logUntil;

	/** publishes new settings */
	public void commitSettings() {
		BankSettings.setInterestRate(interestRate);
		BankSettings.setInterest(interestAmount / 100);
	}

	public void addTransaction() {
		Transaction t = new Transaction();
		Account from = new Account();
		Account to = new Account();

		from.setAccountID(BankSettings.bankAccountID);
		to.setAccountID(Integer.parseInt(recieverAccountID));

		t.setFromAccount(from);
		t.setPaymentReason(paymentReason);
		t.setAmount(amount);
		t.setDate(new Date());
		t.setToAccount(to);
		t.setType(TransactionType.CREDIT);

		try {
			Database.getInstance().addTransaction(t);
		} catch (Throwable e) {
			e.printStackTrace();
		}

	}

	public void filterLog() {
		fillEventLog();
	}

	/** Fills the event log and sorts it descending */
	public void fillEventLog() {
		BufferedReader eventLog;
		String path = DatabaseEventLogger.logPath;

		List<DatabaseLogEntry> logs = new ArrayList<DatabaseLogEntry>();

		for (int i = 1; i <= Integer.MAX_VALUE; i++) {
			try {
				eventLog = new BufferedReader(new FileReader(new File(path)));
			} catch (FileNotFoundException e) {
				break;
			}
			String line = null;
			DatabaseLogEntry entry = null;
			while (true) {
				try {
					line = eventLog.readLine();
					if (line == null)
						break;
					entry = new DatabaseLogEntry(line);
				} catch (IOException e) {
					e.printStackTrace();
				}
				logs.add(entry);
			}
			try {
				eventLog.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			path = DatabaseEventLogger.logPath + "." + i;
		}

		Collections.sort(logs);
		Collections.reverse(logs);

		for (int i = 0; i < logs.size(); i++) {
			if (logFrom != null) {
				if (logs.get(i).getEventTime().before(logFrom)) {
					logs.remove(logs.get(i));
					i--;
				}
			}
		}
		for (int i = 0; i < logs.size(); i++) {
			if (logUntil != null) {
				if (logs.get(i).getEventTime().after(logUntil)) {
					logs.remove(logs.get(i));
					i--;
				}
			}
		}

		setEvents(logs);
	}

	/**
	 * Gets list of {@link Customer}s from the database along with their
	 * {@link Account}s.
	 */
	public void getUserList() {
		try {
			setCustomers(Database.getInstance().getAllCutstomers());
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return current interestRat
	 */
	public InterestRate getInterestRate() {
		return interestRate;
	}

	/**
	 * @param interestRate
	 */
	public void setInterestRate(InterestRate interestRate) {
		this.interestRate = interestRate;
	}

	/**
	 * @return current interestAmount
	 */
	public double getInterestAmount() {
		return interestAmount;
	}

	/**
	 * @param interestAmount
	 */
	public void setInterestAmount(double interestAmount) {
		this.interestAmount = interestAmount;
	}

	/**
	 * @return list of events
	 */
	public List<DatabaseLogEntry> getEvents() {
		return events;
	}

	/**
	 * @param events
	 */
	public void setEvents(List<DatabaseLogEntry> events) {
		this.events = events;
	}

	/**
	 * @return current accountID
	 */
	public String getRecieverAccountID() {
		return recieverAccountID;
	}

	/**
	 * @param recieverAccountID
	 */
	public void setRecieverAccountID(String recieverAccountID) {
		this.recieverAccountID = recieverAccountID;
	}

	/**
	 * @return current reason
	 */
	public String getPaymentReason() {
		return paymentReason;
	}

	/**
	 * @param paymentReason
	 */
	public void setPaymentReason(String paymentReason) {
		this.paymentReason = paymentReason;
	}

	/**
	 * @return amount to send
	 */
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * @param amount
	 */
	public void setAmount(BigDecimal amount) {
		this.amount = amount;
	}

	/**
	 * @return current from date
	 */
	public Date getLogFrom() {
		return logFrom;
	}

	/**
	 * @param logFrom
	 */
	public void setLogFrom(Date logFrom) {
		this.logFrom = logFrom;
	}

	/**
	 * @return current until date
	 */
	public Date getLogUntil() {
		return logUntil;
	}

	/**
	 * @param logUntil
	 */
	public void setLogUntil(Date logUntil) {
		this.logUntil = logUntil;
	}

	/**
	 * @return list of all {@link Customer}s and their {@link Account}s.
	 */
	public List<Customer> getCustomers() {
		return customers;
	}

	/**
	 * @param customers
	 */
	public void setCustomers(List<Customer> customers) {
		this.customers = customers;
	}
	
}
