package database;

import java.util.List;
import java.util.Observable;

import container.Account;
import container.Customer;
import container.Transaction;

/**
 * Abstract class to a database. Can be observed on changes to the database.
 * 
 * @author Carl
 * 
 */
public abstract class AbstractDBController extends Observable {

	/**
	 * Gets a {@link Customer} for given arguments.
	 * 
	 * @param customerID
	 *            The ID of the {@link Customer}.
	 * @param pwd
	 *            The password (Plaintext) of the {@link Customer}.
	 * @return The {@link Customer} or {@code null} if no such {@link Customer}
	 *         exists.
	 * @throws Throwable
	 */
	public abstract Customer getCustomer(int customerID, String pwd)
			throws Throwable;

	/**
	 * Gets a {@link List} of all {@link Customer}s in the database.
	 * 
	 * @return A {@link List} of all {@link Customer}s in the database.
	 * @throws Throwable
	 */
	public abstract List<Customer> getAllCutstomers() throws Throwable;

	/**
	 * Adds a new {@link Customer} to the database. A unique CustomerID will be
	 * generated and returned.
	 * 
	 * @param c
	 *            The {@link Customer} to add. ID will be generated.
	 * @return The ID of the newly added {@link Customer}.
	 * @throws Throwable
	 */
	public abstract int addCustomer(Customer c) throws Throwable;

	/**
	 * Adds a new {@link Account} to the database. A unique AccountID will be
	 * generated and returned.
	 * 
	 * @param a
	 *            The {@link Account} to be added.
	 * @param c
	 *            The {@link Customer} to which this {@link Account} belongs.
	 * @return The ID of the newly added {@link Account}.
	 * @throws Throwable
	 */
	public abstract int addAccount(Account a, Customer c) throws Throwable;

	/**
	 * Tests whether an {@link Account} exists in the database.
	 * 
	 * @param a
	 *            The Account to test.
	 * @return {@code true}, if the {@link Account} exists, {@code false},
	 *         otherwise.
	 * @throws Throwable
	 */
	public abstract boolean isAccount(Account a) throws Throwable;

	/**
	 * Gets a list of all {@link Account}s belonging to a {@link Customer}.
	 * 
	 * @param c
	 *            The {@link Customer}.
	 * @return A List of all accounts which belong to the {@link Customer}.
	 * @throws Throwable
	 */
	public abstract List<Account> getAccounts(Customer c) throws Throwable;

	/**
	 * Gets a List of all {@link Transaction}s involving a specific
	 * {@link Account}.
	 * 
	 * @param a
	 *            The {@link Account} whose {@link Transaction}s to get.
	 * @return A List of all applicable {@link Transaction}s.
	 * @throws Throwable
	 */
	public abstract List<Transaction> getTransactions(Account a)
			throws Throwable;

	/**
	 * Changes a {@link Customer}'s details such as password. If the CustomerID
	 * does not exist, a new {@link Customer} will be added to the database.
	 * 
	 * @param c
	 *            The {@link Customer} which to update.
	 * @throws Throwable
	 */
	public abstract void updateCustomer(Customer c) throws Throwable;

	/**
	 * Adds a {@link Transaction} to the database and commits it.
	 * 
	 * @param t
	 *            The {@link Transaction} to add.
	 * @throws Throwable
	 */
	public abstract void addTransaction(Transaction t) throws Throwable;

	/**
	 * Updates the interest for all {@link Account}s. The rates and when to
	 * update can be set in the {@code BankSettings} class.
	 * 
	 * @throws Throwable
	 */
	public abstract void updateInterest() throws Throwable;

	/**
	 * Notifies the DB if a customers is leaving. Can be used to keep cache up
	 * to date by removing inactive {@link Customer}s from memory.
	 * 
	 * @param customerID
	 *            the leaving customer
	 */
	public abstract void signOff(int customerID);

}
