package database;

import globals.BankSettings;

import java.io.IOException;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import container.Account;
import container.Customer;
import container.Transaction;
import database.helpers.DatabaseEvent;
import database.helpers.DatabaseLogEntry;

/**
 * Observes the database and logs all events to file.
 * 
 * @author Carl
 *
 */
public class DatabaseEventLogger implements Observer {

	/**
	 * File to store events in.
	 */
	public static final String logPath = "db_res/db.log";
	
	/**
	 * Unique logger name.
	 */
	private Logger logger = Logger.getLogger(DatabaseEventLogger.class.getName());
	
	/**
	 * Initialises the logger.
	 */
	public DatabaseEventLogger() {
		try {
			FileHandler fh = new FileHandler(logPath, true);
			fh.setFormatter(new Formatter() {

				@Override
				public String format(LogRecord record) {
					DatabaseLogEntry entry = new DatabaseLogEntry(new Date(record.getMillis()),record.getMessage());
					return entry.toString() + "\n";
				}
				
			});
			logger.addHandler(fh);
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void update(Observable o, Object arg) {
		if (o instanceof AbstractDBController) {
			if (arg instanceof DatabaseEvent) {
				DatabaseEvent event = (DatabaseEvent) arg;
				if (event.getObject() instanceof Customer) {
					Customer c = (Customer) event.getObject();
					switch(event.getEvent()) {
						case CUSTOMER_LOGGED_IN:
							logger.info("Customer " + c.getCustomerID() + " logged in."); break;
						case CUSTOMER_CHANGED:
							logger.info("Customer " + c.getCustomerID() + " changed their details."); break;
						case CUSTOMER_CREATED:
							logger.info("Customer " + c.getCustomerID() + " registered."); break;
						default: break;
					}
				} else if (event.getObject() instanceof Account) {
					Account a = (Account) event.getObject();
					switch(event.getEvent()) {
						case ACCOUNT_CREATED: 
							logger.info("Account " + a.getAccountID() + " was created."); break;
						default: break;
					}
				} else if (event.getObject() instanceof Transaction) {
					Transaction t = (Transaction) event.getObject();
					switch(event.getEvent()) {
						case TRANSACTION_PROCESSED: 
							if (t.getFromAccount().getAccountID() == BankSettings.bankAccountID)
								logger.info("Transaction from E-BANK System to account " + t.getToAccount().getAccountID() + ". Amount: " + t.getAmount().toString() + "."); 
							else
								logger.info("Transaction from account " + t.getFromAccount().getAccountID() + " to account " + t.getToAccount().getAccountID() + ". Amount: " + t.getAmount().toString() + "."); 
							break;
						case TRANSACTION_INTEREST:
							logger.info("Starting interest transactions. (" + BankSettings.getInterest()*100 + "% " + BankSettings.getInterestRate() + ")"); break;
						default: break;
					}
				}
			}
		}
	}

}
