package database;

import globals.BankSettings;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import container.Account;
import container.Customer;
import container.Transaction;
import container.TransactionType;
import database.helpers.DatabaseEvent;
import database.helpers.DatabaseEventType;

/**
 * Proxy for speeding up the reaction of a database. Data of active users
 * is held in memory while they are signed in.
 * 
 * @author Valerius W.
 */
public class DatabaseProxy extends AbstractDBController {
	/**
	 * active accounts
	 */
	private HashMap<Integer, Account> accounts = new HashMap<Integer, Account>();

	/**
	 * active database
	 */
	private AbstractDBController database;

	/**
	 * @param database the active database (can't be null)
	 */
	public DatabaseProxy(AbstractDBController database) {
		if (database == null)
			throw new NullPointerException(
					"DatabaseProxy : database can't be null");

		this.database = database;
	}

	@Override
	public Customer getCustomer(int customerID, String pwd) throws Throwable {
		// put accounts in to hashmap
		Customer c = database.getCustomer(customerID, pwd);
		if (c == null)
			return null;

		for (Account account : c.getAccounts()) {
			accounts.put(account.getAccountID(), account);
		}

		DatabaseEvent e = new DatabaseEvent(c,DatabaseEventType.CUSTOMER_LOGGED_IN);
		setChanged();
		notifyObservers(e);
		clearChanged();
		
		return c;
	}

	@Override
	public List<Customer> getAllCutstomers() throws Throwable {
		return database.getAllCutstomers();
	}
	
	@Override
	public int addCustomer(Customer c) throws Throwable {
		
		int newID =  database.addCustomer(c);
		
		c.setCustomerID(newID);
		
		DatabaseEvent e = new DatabaseEvent(c,DatabaseEventType.CUSTOMER_CREATED);
		setChanged();
		notifyObservers(e);
		clearChanged();
		
		return newID;
	}

	@Override
	public int addAccount(Account a, Customer c) throws Throwable {
		int acID = database.addAccount(a, c);
		accounts.put(acID, a);
		
		DatabaseEvent e = new DatabaseEvent(a,DatabaseEventType.ACCOUNT_CREATED);
		setChanged();
		notifyObservers(e);
		clearChanged();
		
		return acID;
	}

	@Override
	public boolean isAccount(Account a) throws Throwable {
		return database.isAccount(a);
	}

	@Override
	public List<Account> getAccounts(Customer c) throws Throwable {
		if (c.getAccounts() == null) {
			List<Account> accs = database.getAccounts(c);
			c.setAccounts(accs);
			return accs;
		}
		List<Account> accs = new LinkedList<Account>();
		for (Account acc : c.getAccounts()) {
			if (accounts.get(acc.getAccountID()) != null)
				accs.add(accounts.get(acc.getAccountID()));
		}

		return accs;
	}

	@Override
	public List<Transaction> getTransactions(Account a) throws Throwable {
		if (a.getHistory() == null) {
			List<Transaction> history = database.getTransactions(a);
			a.setHistory(history);
			return history;
		} else
			return a.getHistory();
	}

	@Override
	public void updateCustomer(Customer c) throws Throwable {
		database.updateCustomer(c);
		
		DatabaseEvent e = new DatabaseEvent(c,DatabaseEventType.CUSTOMER_CHANGED);
		setChanged();
		notifyObservers(e);
		clearChanged();
		
	}

	@Override
	public void addTransaction(Transaction t) throws Throwable {
		
		database.addTransaction(t);

		t.setDate(new Date());
		if (accounts.containsKey(t.getFromAccount().getAccountID())) {
			if (accounts.get(t.getFromAccount().getAccountID()).getHistory() == null) {
				accounts.get(t.getFromAccount().getAccountID()).setHistory(
						new LinkedList<Transaction>());
				//return;
			}

			BigDecimal balance = accounts
					.get(t.getFromAccount().getAccountID()).getBalance();
			BigDecimal newBalance = balance.add(t.getAmount().negate());
			accounts.get(t.getFromAccount().getAccountID()).setBalance(
					newBalance);

			accounts.get(t.getFromAccount().getAccountID()).getHistory().add(0,t);
		}

		if (accounts.containsKey(t.getToAccount().getAccountID())) {
			if (accounts.get(t.getToAccount().getAccountID()).getHistory() == null) {
				accounts.get(t.getToAccount().getAccountID()).setHistory(
						new LinkedList<Transaction>());
			}

			BigDecimal balance = accounts.get(t.getToAccount().getAccountID())
					.getBalance();
			BigDecimal newBalance = balance.add(t.getAmount());
			accounts.get(t.getToAccount().getAccountID())
					.setBalance(newBalance);

			accounts.get(t.getToAccount().getAccountID()).getHistory().add(0,t);
		}
		
		DatabaseEvent e = new DatabaseEvent(t,DatabaseEventType.TRANSACTION_PROCESSED);
		setChanged();
		notifyObservers(e);
		clearChanged();
	}

	@Override
	public void updateInterest() throws Throwable {
		
		DatabaseEvent e = new DatabaseEvent(new Transaction(),DatabaseEventType.TRANSACTION_INTEREST);
		setChanged();
		notifyObservers(e);
		clearChanged();
		
		database.updateInterest();
		Account from = new Account();
		from.setAccountID(BankSettings.bankAccountID);
		for (Account a : accounts.values()) {
			if (a.getBalance().compareTo(new BigDecimal(0)) > 0) {
				Transaction t = new Transaction();
				t.setDate(new Date());
				t.setAmount(a.getBalance().multiply(
						new BigDecimal(BankSettings.getInterest())));
				t.setToAccount(a);
				t.setFromAccount(from);
				t.setType(TransactionType.CREDIT);
				t.setPaymentReason(BankSettings.getInterestReason());
				if(a.getHistory()==null)
					a.setHistory(new LinkedList<Transaction>());
				a.getHistory().add(t);
			}
		}

	}

	@Override
	public void signOff(int customerID) {
		Customer c = new Customer();
		c.setCustomerID(customerID);
		try {
			List<Account> acc = database.getAccounts(c);
			if (acc != null)
				for (Account account : acc) {
					accounts.remove(account.getAccountID());
				}
		} catch (Throwable e) {
			e.printStackTrace();
		}

	}

}
