package database;

import globals.BankSettings;
import globals.XMLDB;

import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.LinkedList;
import java.util.List;

import com.sleepycat.db.LockMode;
import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlDocumentConfig;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlQueryContext;
import com.sleepycat.dbxml.XmlQueryExpression;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlTransaction;
import com.sleepycat.dbxml.XmlUpdateContext;
import com.sleepycat.dbxml.XmlValue;

import container.Account;
import container.Customer;
import container.Gender;
import container.Transaction;
import container.TransactionType;

import database.helpers.XmlStringFactory;

/**
 * Implementation of a Berkeley XML DB.
 * 
 * @author Carl
 * 
 */
public class XMLDBController extends AbstractDBController {

	private static boolean init = false;

	private enum request {
		UPDATE, INSERT
	}

	/**
	 * Initializes a new XMLDB.
	 * 
	 * @throws XmlException
	 */
	public XMLDBController() throws XmlException {
		if (!init) {
			init = true;
			XMLDB.initContainers();
		}
	}

	@Override
	public Customer getCustomer(int customerID, String pwd) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {
			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			String query = "for $i in collection('" + XMLDB.customerCont
					+ "')/customers/customer\n";
			query += "where $i/customerID = $id and compare($i/password,$pass) = 0\n";
			// query += "order by $i/balance descending\n";
			query += "return (data($i/eMail)," + "data($i/firstName),"
					+ "data($i/familyName)," + "data($i/plz),"
					+ "data($i/city)," + "data($i/street)," + "data($i/tel),"
					+ "data($i/dob)," + "data($i/gender))";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(customerID));
			qc.setVariableValue("pass", new XmlValue(hash(pwd)));

			res = makeQuery(query, qc, mgr);

			if (res.size() == 0)
				return null;

			XmlValue val = new XmlValue();
			Customer c = new Customer();
			DateFormat df = new SimpleDateFormat(XMLDB.datePattern);

			c.setCustomerID(customerID);
			c.setPassword(pwd);

			val = res.next();
			c.seteMail(val.asString());
			val = res.next();
			c.setFirstName(val.asString());
			val = res.next();
			c.setFamilyName(val.asString());
			val = res.next();
			c.setPlz((int) val.asNumber());
			val = res.next();
			c.setCity(val.asString());
			val = res.next();
			c.setStreet(val.asString());
			val = res.next();
			c.setTelephoneNumber((int) val.asNumber());
			val = res.next();
			try {
				c.setDob(df.parse(val.asString()));
			} catch (ParseException e) {
				e.printStackTrace();
			}
			val = res.next();
			if (val.asString().compareTo(Gender.MALE.toString()) == 0)
				c.setGender(Gender.MALE);
			else
				c.setGender(Gender.FEMALE);

			c.setAccounts(getAccounts(c));

			return c;
		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public List<Customer> getAllCutstomers() throws Throwable {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {
			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			String query = "for $i in collection('" + XMLDB.customerCont
					+ "')/customers/customer\n";
			query += "order by $i/customerID ascending\n";
			query += "return (data($i/customerID)," + "data($i/eMail),"
					+ "data($i/firstName)," + "data($i/familyName),"
					+ "data($i/plz)," + "data($i/city)," + "data($i/street),"
					+ "data($i/tel)," + "data($i/dob)," + "data($i/gender))";

			XmlQueryContext qc = mgr.createQueryContext();

			res = makeQuery(query, qc, mgr);

			if (res.size() == 0)
				return null;

			XmlValue val = new XmlValue();
			DateFormat df = new SimpleDateFormat(XMLDB.datePattern);

			List<Customer> customers = new LinkedList<Customer>();
			
			while ((val = res.next()) != null) {
				Customer c = new Customer();
				c.setCustomerID((int) val.asNumber());
				val = res.next();
				c.seteMail(val.asString());
				val = res.next();
				c.setFirstName(val.asString());
				val = res.next();
				c.setFamilyName(val.asString());
				val = res.next();
				c.setPlz((int) val.asNumber());
				val = res.next();
				c.setCity(val.asString());
				val = res.next();
				c.setStreet(val.asString());
				val = res.next();
				c.setTelephoneNumber((int) val.asNumber());
				val = res.next();
				try {
					c.setDob(df.parse(val.asString()));
				} catch (ParseException e) {
					e.printStackTrace();
				}
				val = res.next();
				if (val.asString().compareTo(Gender.MALE.toString()) == 0)
					c.setGender(Gender.MALE);
				else
					c.setGender(Gender.FEMALE);

				c.setAccounts(getAccounts(c));

				customers.add(c);
			}
			
			return customers;
			
		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public int addCustomer(Customer c) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			c.setCustomerID(getNewCustomerID());

			String oldPass = c.getPassword();

			c.setPassword(hash(oldPass));
			String customer = XmlStringFactory.customerXML(c);
			c.setPassword(oldPass);

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setWellFormedOnly(true);

			XmlDocument doc = mgr.createDocument();
			doc.setContent(customer);
			doc.setName(String.valueOf(c.getCustomerID()));

			putDoc(cont, mgr, doc, conf, request.INSERT);

			return c.getCustomerID();
		} finally {
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public int addAccount(Account a, Customer c) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			a.setAccountID(getNewAccountID());

			String account = XmlStringFactory.accountXML(a, c);

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setWellFormedOnly(true);

			XmlDocument doc = mgr.createDocument();
			doc.setContent(account);
			doc.setName(String.valueOf(a.getAccountID()));

			putDoc(cont, mgr, doc, conf, request.INSERT);

			return a.getAccountID();
		} finally {
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public void updateCustomer(Customer c) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			String oldPass = c.getPassword();
			c.setPassword(hash(oldPass));
			String customer = XmlStringFactory.customerXML(c);
			c.setPassword(oldPass);

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setWellFormedOnly(true);

			XmlDocument doc = mgr.createDocument();
			doc.setContent(customer);
			doc.setName(String.valueOf(c.getCustomerID()));

			putDoc(cont, mgr, doc, conf, request.UPDATE);

		} finally {
			XMLDB.cleanup(mgr, cont);
		}
	}

	private int getNewCustomerID() throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			String query = "max(collection('" + XMLDB.customerCont
					+ "')/customers/customer/customerID)";

			XmlQueryContext qc = mgr.createQueryContext();

			res = makeQuery(query, qc, mgr);

			XmlValue val = new XmlValue();

			int id = XMLDB.startCustomerID;
			if ((val = res.next()) != null) {
				id = (int) val.asNumber() + 1;
			}

			return id;

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	private int getNewAccountID() throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			String query = "max(collection('" + XMLDB.accountCont
					+ "')/accounts/account/accountID)";

			XmlQueryContext qc = mgr.createQueryContext();

			res = makeQuery(query, qc, mgr);

			XmlValue val = new XmlValue();

			int id = XMLDB.startAccountID;
			if ((val = res.next()) != null) {
				id = (int) val.asNumber() + 1;
			}

			return id;

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	private int getNewTransactionID() throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.transactionCont);

			String query = "max(collection('" + XMLDB.transactionCont
					+ "')/transactions/transaction/transID)";

			XmlQueryContext qc = mgr.createQueryContext();

			res = makeQuery(query, qc, mgr);

			XmlValue val = new XmlValue();

			int id = 0;
			if ((val = res.next()) != null) {
				id = (int) val.asNumber() + 1;
			}

			return id;

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	@SuppressWarnings("unused")
	private boolean isCustomer(int customerID) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.customerCont);

			String query = "for $i in collection('" + XMLDB.customerCont
					+ "')/customers/customer\n";
			query += "where $i/accountID = $id\n";
			query += "return number($i/customerID)";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(customerID));

			res = makeQuery(query, qc, mgr);

			boolean exists = false;
			if (res.size() > 0)
				exists = true;

			return exists;

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	@Override
	public boolean isAccount(Account a) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			String query = "for $i in collection('" + XMLDB.accountCont
					+ "')/accounts/account\n";
			query += "where $i/accountID = $id\n";
			// query += "order by $i/balance descending\n";
			query += "return number($i/accountID)";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(a.getAccountID()));

			res = makeQuery(query, qc, mgr);

			boolean exists = false;
			if (res.size() > 0)
				exists = true;

			res.delete();

			return exists;
		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public List<Account> getAccounts(Customer c) throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			String query = "for $i in collection('" + XMLDB.accountCont
					+ "')/accounts/account\n";
			query += "where $i/customerID = $id\n";
			// query += "order by $i/balance descending\n";
			query += "return (number($i/accountID)," + "number($i/balance))";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(c.getCustomerID()));

			res = makeQuery(query, qc, mgr);

			XmlValue val = new XmlValue();
			List<Account> list = new LinkedList<Account>();
			Account a = new Account();
			while ((val = res.next()) != null) {
				a = new Account();
				a.setAccountID((int) val.asNumber());
				val = res.next();
				a.setBalance(BigDecimal.valueOf(val.asNumber()));
				list.add(a);
			}

			return list;
		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	private List<Account> getAccounts() throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			String query = "for $i in collection('" + XMLDB.accountCont
					+ "')/accounts/account\n";
			// query += "order by $i/balance descending\n";
			query += "return (number($i/accountID)," + "number($i/balance))";

			XmlQueryContext qc = mgr.createQueryContext();

			res = makeQuery(query, qc, mgr);

			XmlValue val = new XmlValue();
			List<Account> list = new LinkedList<Account>();
			Account a = new Account();
			while ((val = res.next()) != null) {
				a = new Account();
				a.setAccountID((int) val.asNumber());
				val = res.next();
				a.setBalance(BigDecimal.valueOf(val.asNumber()));
				list.add(a);
			}

			return list;
		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	@Override
	public void addTransaction(Transaction t) throws XmlException {
		XmlManager mgr = null;
		XmlContainer cont = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.transactionCont);

			t.setTransID(getNewTransactionID());

			String transaction = XmlStringFactory.transactionXML(t);

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setWellFormedOnly(true);

			XmlDocument doc = mgr.createDocument();
			doc.setContent(transaction);
			doc.setName(String.valueOf(t.getTransID()));
			if (t.getFromAccount().getAccountID() != BankSettings.bankAccountID) {
				if (isAccount(t.getFromAccount())
						&& isAccount(t.getToAccount())) {
					try {
						putDoc(cont, mgr, doc, conf, request.INSERT);
					} catch (XmlException e) {
						if (e.getErrorCode() == XmlException.UNIQUE_ERROR) {
							addTransaction(t);
							return;
						}
					}

					modifyAccount(t.getToAccount(), t.getAmount());
					modifyAccount(t.getFromAccount(), t.getAmount().negate());
				} else {
					throw new IllegalArgumentException(
							"One or both accounts do not exist. ("
									+ t.getFromAccount().getAccountID()
									+ ", and/or "
									+ t.getToAccount().getAccountID() + ")");
				}
			} else {
				try {
					putDoc(cont, mgr, doc, conf, request.INSERT);
				} catch (XmlException e) {
					if (e.getErrorCode() == XmlException.UNIQUE_ERROR) {
						addTransaction(t);
						return;
					}
				}

				modifyAccount(t.getToAccount(), t.getAmount());
			}
		} finally {
			XMLDB.cleanup(mgr, cont);
		}

	}

	private void modifyAccount(Account a, BigDecimal amount)
			throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.accountCont);

			String query = "for $i in collection('" + XMLDB.accountCont
					+ "')/accounts/account\n";
			query += "where $i/accountID = $id\n";
			query += "return (number($i/balance)," + "number($i/customerID))";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(a.getAccountID()));

			res = makeQuery(query, qc, mgr);

			if (res.size() == 0)
				throw new IllegalArgumentException("Account "
						+ a.getAccountID() + "does not exist.");

			XmlValue val = new XmlValue();
			val = res.next();
			Account b = a;
			b.setBalance(new BigDecimal(val.asString()).add(amount));
			val = res.next();
			Customer c = new Customer();
			c.setCustomerID((int) val.asNumber());

			String account = XmlStringFactory.accountXML(b, c);

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setWellFormedOnly(true);

			XmlDocument doc = mgr.createDocument();
			doc.setContent(account);
			doc.setName(String.valueOf(b.getAccountID()));

			putDoc(cont, mgr, doc, conf, request.UPDATE);

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	@Override
	public List<Transaction> getTransactions(Account a) throws XmlException,
			ParseException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.transactionCont);

			String query = "for $i in collection('" + XMLDB.transactionCont
					+ "')/transactions/transaction\n";
			query += "where $i/fromAccountID = $id or $i/toAccountID = $id\n";
			query += "order by $i/date descending\n";
			query += "return (number($i/transID)," + "string($i/date),"
					+ "number($i/fromAccountID)," + "number($i/toAccountID),"
					+ "number($i/amount)," + "string($i/type),"
					+ "string($i/reason))";

			XmlQueryContext qc = mgr.createQueryContext();
			qc.setVariableValue("id", new XmlValue(a.getAccountID()));

			res = makeQuery(query, qc, mgr);

			if (res.size() == 0)
				return null;

			XmlValue val = new XmlValue();
			List<Transaction> ts = new LinkedList<Transaction>();
			Transaction t;
			DateFormat df = new SimpleDateFormat(XMLDB.datePattern);

			while ((val = res.next()) != null) {
				t = new Transaction();
				t.setTransID((int) val.asNumber());
				val = res.next();
				t.setDate(df.parse(val.asString()));
				val = res.next();
				Account from = new Account();
				from.setAccountID((int) val.asNumber());
				t.setFromAccount(from);
				val = res.next();
				Account to = new Account();
				to.setAccountID((int) val.asNumber());
				t.setToAccount(to);
				val = res.next();
				t.setAmount(new BigDecimal(val.asString()));
				val = res.next();
				if (val.asString().compareTo(TransactionType.CREDIT.toString()) == 0)
					t.setType(TransactionType.CREDIT);
				else
					t.setType(TransactionType.DEBITS);
				val = res.next();
				t.setPaymentReason(val.asString());
				ts.add(t);
			}
			return ts;

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}

	}

	public void updateInterest() throws XmlException {

		XmlManager mgr = null;
		XmlContainer cont = null;
		XmlResults res = null;

		try {

			List<Account> accounts = getAccounts();

			mgr = XMLDB.createXmlManager();
			cont = mgr.openContainer(XMLDB.transactionCont);

			Account from = new Account();
			from.setAccountID(BankSettings.bankAccountID);

			for (Account a : accounts) {
				if (a.getBalance().compareTo(new BigDecimal(0)) > 0) {
					Transaction t = new Transaction();
					t.setAmount(a.getBalance().multiply(
							new BigDecimal(BankSettings.getInterest())));
					t.setToAccount(a);
					t.setFromAccount(from);
					t.setType(TransactionType.CREDIT);
					t.setPaymentReason(BankSettings.getInterestReason());
					addTransaction(t);
				}
			}

		} finally {
			if (res != null)
				res.delete();
			XMLDB.cleanup(mgr, cont);
		}
	}

	private void putDoc(XmlContainer cont, XmlManager mgr, XmlDocument doc,
			XmlDocumentConfig conf, request r) throws XmlException {
		XmlTransaction txn = mgr.createTransaction();
		XmlTransaction txn2 = null;
		XmlUpdateContext uc = mgr.createUpdateContext();
		try {
			if (r == request.INSERT) {
				cont.putDocument(txn, doc, uc, conf);
			} else { // r == request.UPDATE
				try {
					cont.putDocument(txn, doc, uc, conf);
				} catch (XmlException e) {
					if (e.getErrorCode() == XmlException.UNIQUE_ERROR) {
						txn2 = mgr.createTransaction();
						XmlDocument newDoc = cont.getDocument(txn2,
								doc.getName());
						newDoc.setContent(doc.getContentAsString());
						cont.updateDocument(txn2, newDoc, uc);
					}
				}
			}
		} finally {
			if (txn2 != null)
				txn2.commit();
			txn.commit();
		}
	}

	private XmlResults makeQuery(String query, XmlQueryContext qc,
			XmlManager mgr) throws XmlException {
		XmlQueryExpression qexpr = null;
		try {
			XmlTransaction txn = mgr.createTransaction();
			XmlDocumentConfig config = new XmlDocumentConfig();
			config.setLockMode(LockMode.RMW);
			XmlResults result = mgr.query(txn, query, qc, config);
			txn.commit();
			return result;
		} finally {
			if (qexpr != null)
				qexpr.delete();
		}
	}

	/**
	 * Hashes a String.
	 * 
	 * @param s
	 * @return SHA-256 hash of the String.
	 */
	private String hash(String s) {
		try {
			MessageDigest md = MessageDigest.getInstance("SHA-256");
			md.update(s.getBytes());

			byte byteData[] = md.digest();

			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < byteData.length; i++) {
				sb.append(Integer.toString((byteData[i] & 0xff) + 0x100, 16)
						.substring(1));
			}
			return sb.toString();

		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return s;
	}

	@Override
	public void signOff(int customerID) {
		// DO NOTHING :)
	}

}
