package globals;

import java.io.File;

import com.sleepycat.db.Environment;
import com.sleepycat.db.EnvironmentConfig;
import com.sleepycat.db.LockDetectMode;
import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlContainerConfig;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlManagerConfig;

/**
 * Class which creates the Berkeley DB XML environment.
 * 
 * @author Carl
 * 
 */
public class XMLDB {
	
	/**
	 * Folder to store the database files in.
	 */
	public static final String databasePath = "db_res/env";
	
	/**
	 * File to store the customer information in.
	 */
	public static final String customerCont = "customer.dbxml";
	
	/**
	 * File to store the account information in.
	 */
	public static final String accountCont = "account.dbxml";
	
	/**
	 * File to store the transaction information in.
	 */
	public static final String transactionCont = "transaction.dbxml";
	
	/**
	 * String format with which to store dates in the database.
	 */
	public static final String datePattern = "yyyy-MM-dd'T'HH:mm:ssz";
	
	/**
	 * Start customer IDs from this number.
	 */
	public static final int startCustomerID = 100000;
	
	/**
	 * Start account IDs from this number.
	 */
	public static final int startAccountID = 100000;
		
	/**
	 * Creates a new manager to access the database.
	 * 
	 * @return new {@link XmlManager}.
	 */
	public static XmlManager createXmlManager() {

		EnvironmentConfig config = new EnvironmentConfig();

		config.setTransactional(true);
		config.setAllowCreate(true);
		//config.setRunRecovery(true);
		config.setInitializeCache(true);
		config.setCacheSize(25 * 1024 * 1024);
		config.setInitializeLocking(true);
		config.setInitializeLogging(true);
		config.setLockDetectMode(LockDetectMode.MINWRITE);
		config.setErrorStream(System.err);

		XmlManagerConfig mconfig = new XmlManagerConfig();
		mconfig.setAllowExternalAccess(true);
		mconfig.setAdoptEnvironment(true);

		XmlContainerConfig contConfig = new XmlContainerConfig();
		contConfig.setContainerType(XmlContainer.NodeContainer);
		contConfig.setIndexNodes(XmlContainerConfig.On);
		contConfig.setTransactional(true);

		Environment env;
		try {
			new File(databasePath).mkdirs();
			env = new Environment(new File(databasePath), config);
			XmlManager mgr = new XmlManager(env, mconfig);
			mgr.setDefaultContainerConfig(contConfig);
			return mgr;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;

	}

	/**
	 * Opens the database or creates a new one if none is found.
	 */
	public static void initContainers() {
		XmlManager mgr = null;
		XmlContainer acc = null;
		XmlContainer cust = null;
		XmlContainer trans = null;

		try {
			mgr = createXmlManager();
			try {
				acc = mgr.openContainer(accountCont);
			} catch (XmlException e) {
				if (e.getErrorCode() == XmlException.CONTAINER_NOT_FOUND)
					acc = mgr.createContainer(accountCont);
				else
					throw e;
			}
			try {
				cust = mgr.openContainer(customerCont);
			} catch (XmlException e) {
				if (e.getErrorCode() == XmlException.CONTAINER_NOT_FOUND)
					cust = mgr.createContainer(customerCont);
				else
					throw e;
			}
			try {
				trans = mgr.openContainer(transactionCont);
			} catch (XmlException e) {
				if (e.getErrorCode() == XmlException.CONTAINER_NOT_FOUND)
					trans = mgr.createContainer(transactionCont);
				else
					throw e;
			}
		} catch (XmlException e) {
			e.printStackTrace();
		} finally {
			cleanup(null, acc);
			cleanup(null, cust);
			cleanup(mgr, trans);
		}
	}

	/*
	private static void fillContainers() {
		XmlManager mgr = null;
		XmlContainer acc = null;
		XmlContainer cust = null;
		XmlContainer trans = null;

		try {
			mgr = createXmlManager();
			acc = mgr.openContainer(accountCont);
			cust = mgr.openContainer(customerCont);
			trans = mgr.openContainer(transactionCont);

			// Fill account.dbxml
			InputStream accountStream = new FileInputStream(new File(
					"db_res/account.xml"));

			XmlInputStream accountsXML = mgr.createInputStream(accountStream);

			XmlDocument accDoc = mgr.createDocument();
			accDoc.setContentAsXmlInputStream(accountsXML);

			XmlUpdateContext accUpdateContext = mgr.createUpdateContext();

			XmlTransaction atxn = mgr.createTransaction();

			XmlDocumentConfig conf = new XmlDocumentConfig();
			conf.setGenerateName(true);
			conf.setWellFormedOnly(true);

			acc.putDocument(atxn, accDoc, accUpdateContext, conf);

			atxn.commit();

			// Fill customer.dbxml
			InputStream customerStream = new FileInputStream(new File(
					"db_res/customer.xml"));

			XmlInputStream customerXML = mgr.createInputStream(customerStream);

			XmlDocument custDoc = mgr.createDocument();
			custDoc.setContentAsXmlInputStream(customerXML);

			XmlUpdateContext custUpdateContext = mgr.createUpdateContext();

			XmlTransaction ctxn = mgr.createTransaction();

			cust.putDocument(ctxn, custDoc, custUpdateContext, conf);

			ctxn.commit();

			// Fill transaction.dbxml
			InputStream transactionStream = new FileInputStream(new File(
					"db_res/transaction.xml"));

			XmlInputStream transactionXML = mgr
					.createInputStream(transactionStream);

			XmlDocument transDoc = mgr.createDocument();
			transDoc.setContentAsXmlInputStream(transactionXML);

			XmlUpdateContext transUpdateContext = mgr.createUpdateContext();

			XmlTransaction ttxn = mgr.createTransaction();

			trans.putDocument(ttxn, transDoc, transUpdateContext, conf);

			ttxn.commit();

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			cleanup(null, acc);
			cleanup(null, cust);
			cleanup(mgr, trans);
		}

	}
	*/

	/**
	 * Delete unneeded objects.
	 * 
	 * @param mgr
	 * @param cont
	 */
	public static void cleanup(XmlManager mgr, XmlContainer cont) {
		try {
			if (cont != null)
				cont.delete();
			if (mgr != null)
				mgr.delete();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
